"""A web3.py Contract class for the ECDSA contract.

DO NOT EDIT.  This file was generated by pypechain v0.0.49.
See documentation at https://github.com/delvtech/pypechain """

# contracts have PascalCase names
# pylint: disable=invalid-name

# contracts control how many attributes and arguments we have in generated code
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-arguments

# we don't need else statement if the other conditionals all have return,
# but it's easier to generate
# pylint: disable=no-else-return

# This file is bound to get very long depending on contract sizes.
# pylint: disable=too-many-lines

# methods are overridden with specific arguments instead of generic *args, **kwargs
# pylint: disable=arguments-differ

# consumers have too many opinions on line length
# pylint: disable=line-too-long

# We use protected classes within pypechain
# pylint: disable=protected-access

# We sometimes define a variable that might not be returned in `call`,
# but we still may want to call the function
# pylint: disable=unused-variable


from __future__ import annotations

from typing import Any, Type, cast

from eth_account.signers.local import LocalAccount
from eth_typing import ABI, ChecksumAddress, HexStr
from hexbytes import HexBytes
from pypechain.core import PypechainBaseContractErrors, PypechainBaseError
from typing_extensions import Self
from web3 import Web3
from web3.contract.contract import (
    Contract,
    ContractConstructor,
    ContractFunctions,
)

structs = {}

ecdsa_abi: ABI = cast(
    ABI,
    [
        {"type": "error", "name": "ECDSAInvalidSignature", "inputs": []},
        {
            "type": "error",
            "name": "ECDSAInvalidSignatureLength",
            "inputs": [
                {"name": "length", "type": "uint256", "internalType": "uint256"}
            ],
        },
        {
            "type": "error",
            "name": "ECDSAInvalidSignatureS",
            "inputs": [
                {"name": "s", "type": "bytes32", "internalType": "bytes32"}
            ],
        },
    ],
)


class ECDSAECDSAInvalidSignatureContractError(PypechainBaseError):
    """ContractError for ECDSAInvalidSignature."""

    # Error name
    name: str = "ECDSAInvalidSignature"
    # 4 byte error selector
    selector: str = "0xf645eedf"
    # error signature, i.e. CustomError(uint256,bool)
    signature: str = "ECDSAInvalidSignature()"
    # Error input types
    abi: ABI = ecdsa_abi


class ECDSAECDSAInvalidSignatureLengthContractError(PypechainBaseError):
    """ContractError for ECDSAInvalidSignatureLength."""

    # Error name
    name: str = "ECDSAInvalidSignatureLength"
    # 4 byte error selector
    selector: str = "0xfce698f7"
    # error signature, i.e. CustomError(uint256,bool)
    signature: str = "ECDSAInvalidSignatureLength(uint256)"
    # Error input types
    abi: ABI = ecdsa_abi


class ECDSAECDSAInvalidSignatureSContractError(PypechainBaseError):
    """ContractError for ECDSAInvalidSignatureS."""

    # Error name
    name: str = "ECDSAInvalidSignatureS"
    # 4 byte error selector
    selector: str = "0xd78bce0c"
    # error signature, i.e. CustomError(uint256,bool)
    signature: str = "ECDSAInvalidSignatureS(bytes32)"
    # Error input types
    abi: ABI = ecdsa_abi


class ECDSAContractErrors(PypechainBaseContractErrors):
    """ContractErrors for the ECDSA contract."""

    ECDSAInvalidSignature: ECDSAECDSAInvalidSignatureContractError

    ECDSAInvalidSignatureLength: ECDSAECDSAInvalidSignatureLengthContractError

    ECDSAInvalidSignatureS: ECDSAECDSAInvalidSignatureSContractError

    def __init__(
        self,
    ) -> None:
        self.ECDSAInvalidSignature = ECDSAECDSAInvalidSignatureContractError()
        self.ECDSAInvalidSignatureLength = (
            ECDSAECDSAInvalidSignatureLengthContractError()
        )
        self.ECDSAInvalidSignatureS = ECDSAECDSAInvalidSignatureSContractError()

        self._all = [
            self.ECDSAInvalidSignature,
            self.ECDSAInvalidSignatureLength,
            self.ECDSAInvalidSignatureS,
        ]


class ECDSAContractFunctions(ContractFunctions):
    """ContractFunctions for the ECDSA contract."""

    def __init__(
        self,
        abi: ABI,
        w3: "Web3",
        address: ChecksumAddress | None = None,
        decode_tuples: bool | None = False,
    ) -> None:
        super().__init__(abi, w3, address, decode_tuples)


class ECDSAContract(Contract):
    """A web3.py Contract class for the ECDSA contract."""

    abi: ABI = ecdsa_abi
    # We change `bytecode` as needed for linking, but keep
    # `_raw_bytecode` unchanged as an original copy.
    # pylint: disable=line-too-long
    _raw_bytecode: HexStr | None = HexStr(
        "0x60556032600b8282823980515f1a607314602657634e487b7160e01b5f525f60045260245ffd5b305f52607381538281f3fe730000000000000000000000000000000000000000301460806040525f80fdfea264697066735822122092544d639e2b5bb52d9fa4080fceb94b8f7dfe0eb275fb73ff6ba72b4b3ebb1864736f6c63430008180033"
    )

    def __init__(self, address: ChecksumAddress | None = None) -> None:
        # Initialize parent Contract class
        super().__init__(address=address)
        self.functions = ECDSAContractFunctions(ecdsa_abi, self.w3, address)  # type: ignore

        self.errors = ECDSAContractErrors()

    errors: ECDSAContractErrors = ECDSAContractErrors()

    functions: ECDSAContractFunctions

    @classmethod
    def constructor(cls) -> ContractConstructor:  # type: ignore
        """Creates a transaction with the contract's constructor function.

        Parameters
        ----------

        w3 : Web3
            A web3 instance.
        account : LocalAccount
            The account to use to deploy the contract.

        Returns
        -------
        Self
            A deployed instance of the contract.

        """
        cls.bytecode = cls._raw_bytecode
        if cls.bytecode is not None:

            # bytecode needs to be in hex for web3
            cls.bytecode = HexBytes(cls.bytecode)

        return super().constructor()

    @classmethod
    def deploy(cls, w3: Web3, account: LocalAccount | ChecksumAddress) -> Self:
        """Deploys an instance of the contract.

        Parameters
        ----------
        w3 : Web3
            A web3 instance.
        account : LocalAccount
            The account to use to deploy the contract.

        Returns
        -------
        Self
            A deployed instance of the contract.
        """
        deployer = cls.factory(w3=w3)
        constructor_fn = deployer.constructor()

        # if an address is supplied, try to use a web3 default account
        if isinstance(account, str):
            tx_hash = constructor_fn.transact({"from": account})
            tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)

            deployed_contract = deployer(address=tx_receipt.contractAddress)  # type: ignore
            return deployed_contract

        # otherwise use the account provided.
        deployment_tx = constructor_fn.build_transaction()
        current_nonce = w3.eth.get_transaction_count(account.address, "pending")
        deployment_tx.update({"nonce": current_nonce})

        # Sign the transaction with local account private key
        signed_tx = account.sign_transaction(deployment_tx)

        # Send the signed transaction and wait for receipt
        tx_hash = w3.eth.send_raw_transaction(signed_tx.raw_transaction)
        tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)

        deployed_contract = deployer(address=tx_receipt.contractAddress)  # type: ignore
        return deployed_contract

    @classmethod
    def factory(
        cls, w3: Web3, class_name: str | None = None, **kwargs: Any
    ) -> Type[Self]:
        """Initializes the contract object.

        Parameters
        ----------
        w3 : Web3
            A web3 instance.
        class_name: str | None
            The instance class name.

        Returns
        -------
        Self
            An instance of the contract class.
        """
        contract = super().factory(w3, class_name, **kwargs)
        contract.functions = ECDSAContractFunctions(ecdsa_abi, w3, None)
        contract.errors = ECDSAContractErrors()

        return contract
